# -*- coding: utf-8 -*-#
#!/usr/bin/env python
"""SQLAlchemy model for the application"""

import sys
if not hasattr(sys, 'frozen'):
    # needed when having multiple versions of SA installed
    import pkg_resources
    pkg_resources.require("sqlalchemy") # get latest version
    
import sqlalchemy as sa
import sqlalchemy.orm as sao
import sqlalchemy.ext.declarative as sad
from sqlalchemy.ext.hybrid import hybrid_property

maker = sao.sessionmaker(autoflush=True, autocommit=False)
DBSession = sao.scoped_session(maker)

class Base(object):
    """Extend the base class

    - Provides a nicer representation when a class instance is printed.
        Found on the SA wiki, not included with TG
    """
    def __repr__(self):
        return "%s(%s)" % (
                 (self.__class__.__name__),
                 ', '.join(["%s=%r" % (key, getattr(self, key))
                            for key in sorted(self.__dict__.keys())
                            if not key.startswith('_')]))

DeclarativeBase = sad.declarative_base(cls=Base)
metadata = DeclarativeBase.metadata

def init_model(engine):
    """Call me before using any of the tables or classes in the model."""
    DBSession.configure(bind=engine)
    
########################################################################
class Person(DeclarativeBase):
    """"""
    __tablename__ = "person"
    
    id = sa.Column(sa.Integer, primary_key=True)
    first_name = sa.Column(sa.Unicode(50))
    last_name = sa.Column(sa.Unicode(50))


########################################################################
class Book(DeclarativeBase):
    """"""
    __tablename__ = "book"
    
    id = sa.Column(sa.Integer, primary_key=True)
    author_id = sa.Column(sa.Integer, sa.ForeignKey("person.id"))
    title = sa.Column(sa.Unicode)
    isbn = sa.Column(sa.Unicode)
    publisher = sa.Column(sa.Unicode)
    person = sao.relationship("Person", backref="books", cascade_backrefs=False)
    
    @hybrid_property
    def author(self):
        return "%s %s" % (self.person.first_name, self.person.last_name)
        